/*
 *===========================================================================
 *
 *   CanHarv.c -  The problem specific definition file for the full
 *	size-structured cannibalistic consumer-resource model with
 *	pulsed reproduction and a single resource: zooplankton.
 *
 *	This model hass been implemented by Tobias van Kooten en used to
 *	generate results as reported in van Kooten et al., 2007, Size-dependent
 *	mortality induces life-history changes mediated through population
 *	dynamical feedbacks. American Naturalist 170: 258-270. The model hence
 *	implements size-dependent harvesting mortality on the cannbialistic
 *	consumer population. 
 *
 *   Last revision: AMdR - Nov 11, 2008
 *
 *==========================================================================
 */
#include  "escbox.h"



/*
 *======================================================================
 *
 *         LABELING ENVIRONMENT AND I-STATE VARIABLES
 *
 *======================================================================
 */

#define time		(env[0])
#define zoop		(env[1])
#define macro		(env[2])
     
#define PERCH		0

#define age		(i_state(0))	// Column 2
#define bone		(i_state(1))	// Column 3
#define fat		(i_state(2))	// Column 4

#define IDmass		(i_const( 0))	// Column 5
#define IDweight	(i_const( 1))	// Column 6
#define IDmaint		(i_const( 2))	// Column 7
#define IDingestz	(i_const( 3))	// Column 8
#define IDingestm	(i_const( 4))	// Column 9
#define IDingestp	(i_const( 5))	// Column 10
#define IDnet_energy	(i_const( 6))	// Column 11
#define IDkappa		(i_const( 7))	// Column 12
#define IDfecundity	(i_const( 8))	// Column 13
#define IDmort		(i_const( 9))	// Column 14
#define IDmortp		(i_const(10))	// Column 15
#define IDfatratio	(i_const(11))	// Column 16
#define IDlength	(i_const(12))	// Column 17
#define IDnbegin	(i_const(13))	// Column 18
#define IDlifefec	(i_const(14))	// Column 19
#define IDscratch	(i_const(15))	// Column 20
#define IDcount		(i_const(16))	// Column 21 
#define IDharvmort	(i_const(17))	// Column 22

#define SMALL		0.001
#define TINY		1.0E-6
#define FOODTINY	1.0E-10


/*
 *======================================================================
 *
 *     DEFEINING AND LABELLING CONSTANTS AND PARAMETERS
 *
 *======================================================================
 */

#define MU_BP		parameter[ 0]	// Background mortality perch
#define MU_SDP		parameter[ 1]	// Size-dependent mortality perch
#define MU_SDEXP	parameter[ 2]	// Size-dependent mortality exponent
#define AR_CONPZ	parameter[ 3]	// Attack rate constant perch on zoop (1).
#define AR_EXPZ		parameter[ 4]	// Attack rate exponent perch on zoop.(1) 
#define AR_WOPTPZ	parameter[ 5]	// Optimal consumpt. weight perch on zoop. (1) 
#define AR_CONPM	parameter[ 6]	// Attack rate constant perch on macro
#define AR_EXPM		parameter[ 7]	// Attack rate exponent perch on macro 
#define AR_CONP		parameter[ 8]	// Attack rate constant piscivory
#define AR_EXP		parameter[ 9]	// Attack rate exponent piscivory
#define DIGT_CONP	parameter[10]	// Digestion time constant perch
#define DIGT_EXPP	parameter[11]	// Digestion time exponent perch
#define MR_CONP		parameter[12]	// Maintenance allometric constant perch
#define MR_EXPP		parameter[13]	// Maintenance allometric exponent perch
#define CONS_IWP	parameter[14]	// Weight of neonate individual perch
#define XMATP		parameter[15]	// Irreversible mass at maturation perch
#define QJP		parameter[16]	// Juvenile y/x ratio perch
#define QAP		parameter[17]	// Adult y/x ratio perch
#define CONVEFFP	parameter[18]	// Food conversion efficiency perch
#define QSP		parameter[19]	// Starvation mortality threshold perch
#define MU_SP		parameter[20]	// Starvation mortality parameter perch
#define L_CONP		parameter[21]	// Length allometric constant perch
#define L_EXPP		parameter[22]	// Length allometric exponent perch
#define QOPT		parameter[23]	// Optimum y/x ratio for piscivory
#define QMIN		parameter[24]	// Minimum y/x ratio for piscivory
#define QMAX		parameter[25]	// Maximum y/x ratio for piscivory
#define RZOOP		parameter[26]	// Growth rate zooplankton
#define RMACRO		parameter[27]	// Growth rate macro-invertebrates
#define KZOOP		parameter[28]	// Carrying capacity zooplankton
#define KMACRO		parameter[29]	// Carrying capacity macro-invertebrates
#define LAKEVOL		parameter[30]	// Pelagic lake volume
#define LAKEBOT		parameter[31]	// Littoral lake bottom surface
#define YEAR		parameter[32]	// Growth season duration
#define HARV_MIN	parameter[33]	// Harvesting mortality lower size threshold
#define MU_HARV		parameter[34]	// Harvesting mortality rate
#define HARV_MAX	parameter[35]	// Harvesting mortality upper size threshold



/*
 *===========================================================================
 *
 *      DEFINING GLOBAL VARIABLES AND USER-IMPLEMENTED FUNCTIONS
 *
 *===========================================================================
 */

static void     UpdateIDcards(double *env, population *pop, int setfec);


int			ReproCohorts;
double			TotalRepro;
double			MeanFecundity;
int			countindex;


/*

 *===========================================================================
 *
 * USER INITIALIZATION ROUTINE ALLOWS OPERATIONS ON INITIAL POPULATIONS
 *
 *===========================================================================
 */

void    UserInit(int argc, char **argv, double *env, population *pop)   
{
  register int		v;

  switch (argc)
    {
    case 3:
      MU_SDP = atof(argv[3]);
    default:
      break;
    }

  UpdateIDcards(env, pop, 0);

  // Assign cohort count index to initial cohorts

  for (countindex=0;  countindex<cohort_no[PERCH]; countindex++) 
    {
      popIDcard[PERCH][countindex][IDcount] = countindex;
    }

  return;
}



/*
 *===========================================================================
 *
 *  SPECIFICATION OF THE NUMBER AND VALUES OF BOUNDARY POINTS
 *
 *===========================================================================
 */

void    SetBpointNo(double *env, population *pop, int *bpoint_no)
{
  bpoint_no[PERCH] = 0; 

  return;
}



/*===========================================================================*/

void    SetBpoints(double *env, population *pop, population *bpoints)
{

  return;
}



/*
 *=====================================================================
 *
 *           SPECIFICATION OF DERIVATIVES
 *
 *=====================================================================
 */

void    Gradient(double *env,     population *pop,     population *ofs,
		 double *envgrad, population *popgrad, population *ofsgrad,
		 population *bpoints)
{
  register int		i;
  double		kappa = 0.0, net_energy = 0.0;
  double		tot_grazingz=0.0, tot_grazingm= 0.0;
  double		mort = 0.0, dummy;

  UpdateIDcards(env, pop, 0);

  // Perch derivatives
  for (i=0; i<cohort_no[PERCH]; i++)
    {
      popgrad[PERCH][i][number] =  0.0;
      popgrad[PERCH][i][age]    =  1.0;
      popgrad[PERCH][i][bone]   =  0.0;
      popgrad[PERCH][i][fat]    = -1.0;
      if (pop[PERCH][i][fat] < TINY*pop[PERCH][i][bone]) continue;

      mort       = popIDcard[PERCH][i][IDmort] + popIDcard[PERCH][i][IDmortp];
      kappa      = popIDcard[PERCH][i][IDkappa];
      net_energy = popIDcard[PERCH][i][IDnet_energy];

      popgrad[PERCH][i][number] = -mort*pop[PERCH][i][number];
      popgrad[PERCH][i][bone]   = kappa*net_energy;
      popgrad[PERCH][i][fat]    = (1-kappa)*net_energy;

      tot_grazingz += popIDcard[PERCH][i][IDingestz]*pop[PERCH][i][number];
      tot_grazingm += popIDcard[PERCH][i][IDingestm]*pop[PERCH][i][number];

    }

  // Environment derivatives
  envgrad[0] = 1.0;
  envgrad[1] = RZOOP * (exp(KZOOP)-zoop) - tot_grazingz/LAKEVOL;
  envgrad[2] = RMACRO * (exp(KMACRO)-macro) - tot_grazingm/LAKEBOT;

  return;
}



/*
 *===========================================================================
 *
 *      SPECIFICATION OF BETWEEN COHORT CYCLE DYNAMICS
 *
 *===========================================================================
 */

void    InstantDynamics(double *env, population *pop, population *ofs)
{
  register int		i, newcoh;
  double		juveniles, dummy;

  if (zoop  < FOODTINY) zoop  = FOODTINY;	// Check negative food
  if (macro < FOODTINY) macro = FOODTINY;


  for (i=0; i<cohort_no[PERCH];i++)	// Remove cohorts with no fat
    {
      if (pop[PERCH][i][fat] < TINY*pop[PERCH][i][bone])
	pop[PERCH][i][number] = 0.0;
    }

  dummy = fmod(time, YEAR);

  // Within a year, no reproduction
  if (( dummy > SMALL) && (dummy < (YEAR - SMALL))) return;

  UpdateIDcards(env, pop, 1);


  // Perch reproduction

  ReproCohorts = 0.0;
  TotalRepro   = 0.0;
  juveniles    = 0.0;

  for (i=0; i<cohort_no[PERCH]; i++)
    {
      if (pop[PERCH][i][fat] < TINY*pop[PERCH][i][bone]) continue;

      if (pop[PERCH][i][bone] > XMATP)
	{
	  if (popIDcard[PERCH][i][IDfecundity] > 0.0)
	    {
	      juveniles += (popIDcard[PERCH][i][IDfecundity]*pop[PERCH][i][number]);
	      pop[PERCH][i][fat] = QJP*pop[PERCH][i][bone];
	      ReproCohorts++;
	      TotalRepro += pop[PERCH][i][number];
	    }
	}
    }

  if (juveniles)
    {
      newcoh = AddCohorts(pop, PERCH, 1);
      pop[PERCH][newcoh][number] = juveniles;
      pop[PERCH][newcoh][age]    = 0.0;
      pop[PERCH][newcoh][bone]   = 1/(1+QJP)*CONS_IWP;
      pop[PERCH][newcoh][fat]    = (QJP)/(1+QJP)*CONS_IWP;
      popIDcard[PERCH][newcoh][IDnbegin]    = juveniles;
      popIDcard[PERCH][newcoh][IDfecundity] = 0.0;
      popIDcard[PERCH][newcoh][IDlifefec]   = 0.0;

      ++countindex;
      popIDcard[PERCH][newcoh][IDcount] = countindex;

      SetStepSize(SMALL);
    }
  if (TotalRepro) MeanFecundity = juveniles/TotalRepro;
  else MeanFecundity = 0.0;

  return;
}         


/*
 *===========================================================================
 *
 *          SPECIFICATION OF OUTPUT VARIABLES
 *
 *===========================================================================
 */

void    DefineOutput(double *env, population *pop, double *output)
{
  register int         i;
  double               num, biom;

  /* OUTPUT VARIABLES:
   * [ 0]   2: Zooplankton density
   * [ 1]   3: Macro-invertebrates density
   * [ 2]   4: Perch YOY number
   * [ 3]   5: Perch YOY biomass 
   * [ 4]   6: Perch 1+ number
   * [ 5]   7: Perch 1+ biomass
   * [ 6]   8: Perch juv number
   * [ 7]   9: Perch juv biomass
   * [ 8]  10: Perch adult number
   * [ 9]  11: Perch adult biomass
   * [10]  12: Perch length of oldest cohort
   * [11]  13: Perch lifetime repro-output
   * [12]  14: Piscivory mortality on perch
   * [13]  15: Ingest zoopl perch
   * [14]  16: Ingest macro perch
   * [15]  17: Ingest pisc perch
   * [16]  18: Harvested biomass
   */

  UpdateIDcards(env, pop, 0);

#if (BIFURCATION == 1)
  LabelState(PERCH, "par. = %.4f   T = %.0f", parameter[BifParIndex], env[0]);
#endif // BIFURCATION

  output[ 0] = zoop;
  output[ 1] = macro;

  for (i=0; i<cohort_no[PERCH]; i++)
    {
      num  = pop[PERCH][i][number];
      biom = (pop[PERCH][i][bone]+pop[PERCH][i][fat])*pop[PERCH][i][number];

      if (pop[PERCH][i][age]< (YEAR - SMALL))	// YOY
        {
	  output[2] += num;
	  output[3] += biom;
        }
      else					// 1+
	{
	  output[4] += num;
	  output[5] += biom;
	}
      if(pop[PERCH][i][bone] < XMATP) 		// Juveniles
	{
	  output[6] += num;
	  output[7] += biom;
	}
      else					// Adults
	{
	  output[8] += num;
	  output[9] += biom;
	}

      if (i == 0)
        {
	  output[10]  = popIDcard[PERCH][i][IDlength];     
	  output[11] = popIDcard[PERCH][i][IDlifefec];
        }

      output[12] += popIDcard[PERCH][i][IDmortp]; 
      output[13] += popIDcard[PERCH][i][IDingestz]*num; //Ingest zoopl perch
      output[14] += popIDcard[PERCH][i][IDingestm]*num; //Ingest macro perch
      output[15] += popIDcard[PERCH][i][IDingestp]*num; //Ingest pisc perch

      if((popIDcard[PERCH][i][IDlength]>HARV_MIN)&&(popIDcard[PERCH][i][IDlength]<HARV_MAX))
        output[16] += popIDcard[PERCH][i][IDharvmort]*(pop[PERCH][i][bone]+pop[PERCH][i][fat]);
    }       

   return;
}

/*==============================================================================*/

static void UpdateIDcards(double *env, population *pop, int setfec)

{
  register int		i, p, r;
  double		mass, weight, fatratio, l_ratio;
  double		aratez, mass_encz, ingestz;
  double		aratem, mass_encm, ingestm;
  double		aconp, aratep, mass_encp, ingestp;
  double		htime, gutspace;
  double		maint, net_energy, kappa, q;
  double		fecundity, mort, handling, eta;

  for (i=0; i<cohort_no[PERCH]; i++)
    {
      popIDcard[PERCH][i][IDmass]       = pop[PERCH][i][bone]*(1+QJP);
      popIDcard[PERCH][i][IDweight]     = pop[PERCH][i][bone] + pop[PERCH][i][fat];
      popIDcard[PERCH][i][IDfatratio]   = pop[PERCH][i][fat]/pop[PERCH][i][bone];
      popIDcard[PERCH][i][IDlength]     = L_CONP*pow(popIDcard[PERCH][i][IDmass], L_EXPP);

      popIDcard[PERCH][i][IDmaint]      = 0.0;
      popIDcard[PERCH][i][IDingestz]    = 0.0;
      popIDcard[PERCH][i][IDingestm]    = 0.0;
      popIDcard[PERCH][i][IDingestp]    = 0.0;
      popIDcard[PERCH][i][IDnet_energy] = 0.0;
      popIDcard[PERCH][i][IDkappa]      = 0.0;
      popIDcard[PERCH][i][IDmort]       = 0.0;
      popIDcard[PERCH][i][IDmortp]      = 0.0;
      popIDcard[PERCH][i][IDharvmort]   = 0.0;
    }

  for (i=0; i<cohort_no[PERCH]; i++)
    {
      mass      = popIDcard[PERCH][i][IDmass];
      fatratio  = popIDcard[PERCH][i][IDfatratio];
      if (fatratio <= TINY) continue;

      maint     = MR_CONP*pow(popIDcard[PERCH][i][IDweight], MR_EXPP);
      htime     = DIGT_CONP*pow(mass, DIGT_EXPP);

      // Perch attack and ingest on zooplankton
      aratez    = AR_CONPZ*pow(mass/AR_WOPTPZ*exp(1-mass/AR_WOPTPZ), AR_EXPZ);    
      mass_encz = aratez*zoop;

      // Perch attack and ingest on macro-invertebrates
      aratem    = AR_CONPM*pow(mass, AR_EXPM);
      mass_encm = aratem*macro;

      // General piscivorous perch attack rate based on length
      aconp     =  AR_CONP*pow(popIDcard[PERCH][i][IDlength], AR_EXP)/LAKEVOL;

      // Perch attack and ingest on perch
      mass_encp = 0.0;
      for ( p=0; p<cohort_no[PERCH]; p++ )
	{
          popIDcard[PERCH][p][IDscratch] = 0.0;
          if (popIDcard[PERCH][p][IDfatratio] <= TINY) continue;

          l_ratio = (popIDcard[PERCH][p][IDlength]/popIDcard[PERCH][i][IDlength]);
          if ((l_ratio > QMIN ) && (l_ratio <QMAX))
	    {
              if (l_ratio <= QOPT)
		aratep = (aconp * (l_ratio - QMIN )/(QOPT -QMIN));
              else
		aratep = (aconp * (QMAX - l_ratio)/(QMAX -QOPT));

              mass_encp += aratep*(pop[PERCH][p][number]*popIDcard[PERCH][p][IDweight]);
              popIDcard[PERCH][p][IDscratch] = aratep*pop[PERCH][i][number];
	    }
	}

      // Total ingestion and netto energy perch
      gutspace = 1+htime*(mass_encz+mass_encm+mass_encp);

      ingestz  = mass_encz/gutspace;
      ingestm  = mass_encm/gutspace;
      ingestp  = mass_encp/gutspace;

      for (p=0; p<cohort_no[PERCH]; p++)
	popIDcard[PERCH][p][IDmortp] += popIDcard[PERCH][p][IDscratch]/gutspace; 

      net_energy = (CONVEFFP*(ingestz + ingestm + ingestp) - maint);

      if (pop[PERCH][i][bone] < XMATP) q = QJP;
      else q = QAP;
      if (net_energy > 0.0)
	{
          if (fatratio > q) kappa = 1/(q+1);
          else kappa = fatratio/((1+q)*q);
	}
      else kappa = 0.0;

      mort  = MU_BP;
      mort += MU_SDP*exp(-pop[PERCH][i][bone]/MU_SDEXP);

      if((popIDcard[PERCH][i][IDlength]>HARV_MIN)&&(popIDcard[PERCH][i][IDlength]<HARV_MAX))
	{
          mort += MU_HARV;
          popIDcard[PERCH][i][IDharvmort]+=MU_HARV*pop[PERCH][i][number];
	}

      if (fatratio < QSP) mort += MU_SP*(QSP/fatratio - 1.0);

      if (setfec)
	{
          if ((pop[PERCH][i][bone] > XMATP) && (fatratio > QJP))
	    {
              fecundity = (pop[PERCH][i][fat]-QJP*pop[PERCH][i][bone])/(2.0*CONS_IWP);
              popIDcard[PERCH][i][IDlifefec] += (fecundity*pop[PERCH][i][number]/popIDcard[PERCH][i][IDnbegin]);
	    }
          else fecundity = 0.0;
          popIDcard[PERCH][i][IDfecundity] = fecundity;
	}

      popIDcard[PERCH][i][IDmaint]      = maint;
      popIDcard[PERCH][i][IDingestz]    = ingestz;
      popIDcard[PERCH][i][IDingestm]    = ingestm;
      popIDcard[PERCH][i][IDingestp]    = ingestp;
      popIDcard[PERCH][i][IDnet_energy] = net_energy;
      popIDcard[PERCH][i][IDkappa]      = kappa;
      popIDcard[PERCH][i][IDmort]       = mort;
    }

  return;
}

/*=============================================================================*/



